# == Parameters:
#
# [*api_key_pem*]
#   (Required) base64 encoded file for Key + Certificate of the API.
#
class supply::services::haproxy(
  $machine_ip                   = undef,
  $controller_ips               = [],
  $controller_hostnames         = [],

  $api_key_pem                  = undef,
  $pass_haproxy_stats           = undef,

  $haproxy_timeout_http_request = '10s',
  $haproxy_timeout_queue        = '1m',
  $haproxy_timeout_connect      = '10s',
  $haproxy_timeout_client       = '1m',
  $haproxy_timeout_server       = '1m',
  $haproxy_timeout_check        = '10s',
){

  file { "/etc/haproxy/ssl":
    ensure                  => directory,
    owner                   => 'root',
    mode                    => '0755',
    selinux_ignore_defaults => true,
    require                 => Package['haproxy'],
  }->
  file { "/etc/haproxy/ssl/private":
    ensure                  => directory,
    owner                   => 'root',
    mode                    => '0755',
    selinux_ignore_defaults => true,
  }->
  file { "/etc/haproxy/ssl/private/supply-baremetal-api.pem":
    ensure                  => present,
    owner                   => "haproxy",
    content                 => base64('decode', $api_key_pem),
    selinux_ignore_defaults => true,
    mode                    => '0600',
    notify                  => Service['haproxy'],
  }

  class { 'haproxy':
    global_options   => {
      'log'     => '/dev/log local0',
      'chroot'  => '/var/lib/haproxy',
      'pidfile' => '/var/run/haproxy.pid',
      'maxconn' => '40960',
      'user'    => 'haproxy',
      'group'   => 'haproxy',
      'stats'   => [
        'socket /var/lib/haproxy/stats',
        'timeout 30s'
      ],
      'daemon'   => '',
      'nbthread' => '8',
      'ssl-default-bind-ciphers'   => 'ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256',
      'ssl-default-bind-options'   => 'no-sslv3 no-tlsv10 no-tlsv11 no-tls-tickets',
      'ssl-default-server-ciphers' => 'ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256',
      'ssl-default-server-options' => 'no-sslv3 no-tlsv10 no-tlsv11 no-tls-tickets',
    },
    defaults_options => {
      'log'     => 'global',
      'mode'      => 'http',
      'option'    => [
          'httplog',
        ],
      'retries' => '3',
      'timeout'   => [
          "http-request ${haproxy_timeout_http_request}",
          "queue ${haproxy_timeout_queue}",
          "connect ${haproxy_timeout_connect}",
          "client ${haproxy_timeout_client}",
          "server ${haproxy_timeout_server}",
          "check ${haproxy_timeout_check}",
        ],
      'maxconn' => '8000',
      'monitor-uri' => '/health'
    },
    merge_options => false,
# TODO: Fix scheduling.
#    before        => Anchor[$haproxy_schedule],
    require       => Sysctl::Value['net.ipv4.ip_nonlocal_bind'],
  }->
  haproxy::listen { 'hapstats':
    section_name => 'statshttp',
    bind         => { "${machine_ip}:8088" => 'user root'},
    mode         => 'http',
    options      => {
        'stats' => [ 'uri /', "auth admin:${pass_haproxy_stats}", 'refresh 15', 'admin if TRUE', 'realm Haproxy Statistics', ],
      },
  }->
  haproxy::frontend { 'health_check':
    section_name => 'healthcheck',
    bind         => { "${machine_ip}:8081" => 'user root'},
    mode         => 'http',
    options      => {
      timeout      => 'client 5s',
      monitor-uri  => '/',
    }
  }

  logrotate::rule { 'haproxy':
    path          => '/var/log/haproxy.log',
    rotate        => '7',
    rotate_every  => 'day',
    missingok     => true,
    compress      => true,
    delaycompress => true,
    postrotate    => '/usr/lib/rsyslog/rsyslog-rotate',
  }

  haproxy::frontend { 'openstackfe':
    mode      => 'http',
    bind      => {
                   "${machine_ip}:443" => ['ssl', 'crt', '/etc/haproxy/ssl/private/'],
                 },
    options   => [
                   { 'http-response' => 'set-header Strict-Transport-Security max-age=63072000'},
                   { 'option'        => "forwardfor except ${machine_ip}"},
                   { 'acl'           => 'url_keystone path_beg -i /identity'},
                   { 'use_backend'   => 'keystonebe if url_keystone'},
#                   { 'acl'           => 'url_glance path_beg -i /image'},
#                   { 'use_backend'   => 'glancebe if url_glance'},
#                   { 'acl'           => 'url_horizon path_beg -i /horizon'},
#                   { 'use_backend'   => 'horizonbe if url_horizon'},
#                   { 'acl'           => 'url_nova path_beg -i /compute'},
#                   { 'use_backend'   => 'novabe if url_nova'},
#                   { 'acl'           => 'url_neutron path_beg -i /network'},
#                   { 'use_backend'   => 'neutronbe if url_neutron'},
#                   { 'acl'           => 'url_cinder path_beg -i /volume'},
#                   { 'use_backend'   => 'cinderbe if url_cinder'},
#                   { 'acl'           => 'url_gnocchi path_beg -i /metric'},
#                   { 'acl'           => 'url_gnocchi path_beg -i /v1/metric'},
#                   { 'use_backend'   => 'gnocchibe if url_gnocchi'},
#                   { 'acl'           => 'url_cloudkitty path_beg -i /rating'},
#                   { 'use_backend'   => 'cloudkittybe if url_cloudkitty'},
#                   { 'acl'           => 'url_aodh path_beg -i /alarm'},
#                   { 'use_backend'   => 'aodhbe if url_aodh'},
#                   { 'acl'           => 'url_octavia path_beg -i /loadbalance'},
#                   { 'use_backend'   => 'octaviabe if url_octavia'},
#                   { 'acl'         => 'url_magnum path_beg -i /containers'},
#                   { 'use_backend' => 'magnumbe if url_magnum'},
#                   { 'acl'         => 'url_etcd_cluster path_beg -i /etcd-cluster'},
#                   { 'use_backend' => 'etcdclusterbe if url_etcd_cluster'},
#                   { 'acl'         => 'url_etcd_discovery path_beg -i /etcd-discovery'},
#                   { 'use_backend' => 'etcddiscoverybe if url_etcd_discovery'},
#                   { 'acl'         => 'url_designate path_beg -i /dns'},
#                   { 'use_backend' => 'designatebe if url_designate'},
#                   { 'acl'         => 'url_manila path_beg -i /share'},
#                   { 'use_backend' => 'manilabe if url_manila'},
#                   { 'acl'         => 'url_ironic path_beg -i /baremetal'},
#                   { 'use_backend' => 'ironicbe if url_ironic'},
#                   { 'acl'         => 'url_ironicinspector path_beg -i /inspector'},
#                   { 'use_backend' => 'ironicinspectorbe if url_ironicinspector'},
                 ],
  }

  haproxy::backend { 'keystonebe':
    options => [
       { 'option'                       => 'forwardfor' },
       { 'option'                       => 'httpchk GET /healthcheck' },
       { 'http-request'                 => "set-header X-Client-IP %[src]"},
       { 'mode'                         => 'http' },
       { 'balance'                      => 'roundrobin' },
       { 'http-request replace-path'    => '^/identity/?(.*) /\1' },
       { 'http-response replace-header' => 'Location: (https://[^/]*)/(.*)$ \1/identity/\2' },
    ],
  }
  haproxy::balancermember { 'keystonepubbm':
    listening_service => 'keystonepubbe',
    ipaddresses       => $controller_ips,
    server_names      => $controller_hostnames,
    ports             => 5000,
    options           => 'check check-ssl ssl verify none',
#    options           => 'check check-ssl ssl verify required ca-file /etc/ssl/certs/supply-pki-api.pem',
  }

}
